package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.params.TeachCourseSelectParam;
import com.qmth.distributed.print.business.bean.result.CourseResult;
import com.qmth.distributed.print.business.bean.result.CourseWeightSettingStatusResult;
import com.qmth.distributed.print.business.bean.result.DictionaryResult;
import com.qmth.distributed.print.business.bean.result.TeachCourseResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.CourseSettingTypeEnum;
import com.qmth.distributed.print.business.mapper.TeachCourseMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.OrgTypeEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 教学课程服务实现类
 * @Author: CaoZixuan
 * @Date: 2022-02-07
 */
@Service
public class TeachCourseServiceImpl extends ServiceImpl<TeachCourseMapper, TeachCourse> implements TeachCourseService {

    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private TeachClazzService teachClazzService;
    @Resource
    private CourseTargetService courseTargetService;
    @Resource
    private CourseEvaluationService courseEvaluationService;
    @Resource
    private CourseWeightService courseWeightService;
    @Resource
    private CourseDimensionService courseDimensionService;
    @Resource
    private TeachStudentService teachStudentService;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;

    @Override
    public IPage<TeachCourseResult> teachCoursePage(Long examId, String courseName, int pageNumber, int pageSize) {
        courseName = SystemConstant.translateSpecificSign(courseName);
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(requestUser.getSchoolId(),
                requestUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.findTeachCoursePage(new Page<>(pageNumber, pageSize), examId, courseName, requestUser.getSchoolId(), dpr);
    }

    @Transactional
    @Override
    public void selectCreate(TeachCourseSelectParam teachCourseSelectParam) {
        Long examId = teachCourseSelectParam.getExamId();
        List<Long> courseIds = teachCourseSelectParam.getCourseIds();

        List<Long> userCourseIds = this.list(
                        new QueryWrapper<TeachCourse>().lambda().eq(TeachCourse::getExamId, examId).eq(TeachCourse::getUserId, teachCourseSelectParam.getUserId())).stream().map(TeachCourse::getCourseId)
                .collect(Collectors.toList());

        List<TeachCourse> teachCourseList = new ArrayList<>();
        for (Long courseId : courseIds) {
            if (CollectionUtils.isNotEmpty(courseIds) && userCourseIds.contains(courseId)) {
                continue;
            }
            TeachCourse teachCourse = new TeachCourse();
            teachCourse.setSchoolId(teachCourseSelectParam.getSchoolId());
            teachCourse.setExamId(examId);
            teachCourse.setUserId(teachCourseSelectParam.getUserId());
            teachCourse.setEnable(true);
            teachCourse.setWeightSetting(false);
            teachCourse.setCourseId(courseId);
            teachCourse.insertInfo(teachCourseSelectParam.getUserId());
            teachCourseList.add(teachCourse);
        }

        if (CollectionUtils.isNotEmpty(teachCourseList)) {
            this.saveBatch(teachCourseList);

            for (TeachCourse teachCourse : teachCourseList) {
                // 创建默认评价方式
                courseEvaluationService.createDefaultCourseEvaluation(teachCourse.getId());
            }
        }
    }

    @Override
    public List<DictionaryResult> findBasicCourseByUser(SysUser requestUser) {
        Long userId = requestUser.getId();
        Set<Long> orgIdSet = sysOrgService.findDeepOrgIdListByUserId(userId);
        QueryWrapper<BasicCourse> basicCourseQueryWrapper = new QueryWrapper<>();
        basicCourseQueryWrapper.lambda().eq(BasicCourse::getEnable, true);
        if (orgIdSet.size() > 0) {
            basicCourseQueryWrapper.lambda().in(BasicCourse::getTeachingRoomId, orgIdSet);
        }
        List<BasicCourse> basicCourseList = basicCourseService.list(basicCourseQueryWrapper);

        List<DictionaryResult> resultList = new ArrayList<>();
        for (BasicCourse basicCourse : basicCourseList) {
            DictionaryResult dictionaryResult = new DictionaryResult();
            BeanUtils.copyProperties(basicCourse, dictionaryResult);
            resultList.add(dictionaryResult);
        }
        return resultList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteTeachCourseBatch(List<Long> idList) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long requestUserId = requestUser.getId();
        List<TeachCourse> teachCourseList = this.listByIds(idList);
        for (Long teachCourseId : idList) {
            // 删除知识点
            courseDimensionService.remove(
                    new QueryWrapper<CourseDimension>().lambda().eq(CourseDimension::getTeachCourseId, teachCourseId));
            // 删除课程目标
            courseTargetService.remove(
                    new QueryWrapper<CourseTarget>().lambda().eq(CourseTarget::getTeachCourseId, teachCourseId));
            // 删除课程评价方式
            courseEvaluationService.remove(new QueryWrapper<CourseEvaluation>().lambda()
                    .eq(CourseEvaluation::getTeachCourseId, teachCourseId));
            // 删除权重设置
            courseWeightService.remove(
                    new QueryWrapper<CourseWeight>().lambda().eq(CourseWeight::getTeachCourseId, teachCourseId));

            // 删除教学学生
            teachStudentService.remove(
                    new QueryWrapper<TeachStudent>().lambda().eq(TeachStudent::getTeachCourseId, teachCourseId));
            // 删除教学班
            teachClazzService.remove(
                    new QueryWrapper<TeachClazz>().lambda().eq(TeachClazz::getTeachCourseId, teachCourseId).or()
                            .eq(TeachClazz::getBelongTeachCourseId, teachCourseId));

            // 删除教学课程
            this.removeById(teachCourseId);
        }
    }

    @Override
    public TeachCourse findByExamIdCourseIdAndUserId(Long examId, Long courseId, Long userId) {
        TeachCourse teachCourse = this.getOne(
                new QueryWrapper<TeachCourse>().lambda().eq(TeachCourse::getExamId, examId).eq(TeachCourse::getCourseId, courseId).eq(TeachCourse::getUserId, userId)
                        .last(SystemConstant.LIMIT1));
        if (Objects.isNull(teachCourse)) {
            throw ExceptionResultEnum.ERROR.exception("教学课程不存在");
        }
        return teachCourse;
    }

    @Transactional
    @Override
    public void clearCourseSetting(Long teachCourseId, CourseSettingTypeEnum courseSettingTypeEnum) {
        if (courseSettingTypeEnum == CourseSettingTypeEnum.COURSE_DIMENSION) {// 删除知识点、目标设置
            SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
            Long requestUserId = requestUser.getId();

            courseDimensionService.remove(
                    new QueryWrapper<CourseDimension>().lambda().eq(CourseDimension::getTeachCourseId, teachCourseId));

            UpdateWrapper<CourseTarget> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(CourseTarget::getTeachCourseId, teachCourseId)
                    .set(CourseTarget::getTargetContent, null).set(CourseTarget::getUpdateId, requestUserId)
                    .set(CourseTarget::getUpdateTime, System.currentTimeMillis());
            courseTargetService.update(updateWrapper);

            UpdateWrapper<TeachCourse> teachCourseUpdateWrapper = new UpdateWrapper<>();
            teachCourseUpdateWrapper.lambda().eq(TeachCourse::getId, teachCourseId)
                    .set(TeachCourse::getWeightSettingSign, SystemConstant.getDbUuid())
                    .set(TeachCourse::getDimensionSign, SystemConstant.getDbUuid());
            this.update(teachCourseUpdateWrapper);
        } else {
            // 删除权重设置
            courseWeightService.deleteCourseWeight(teachCourseId);
        }
    }

    @Override
    public CourseWeightSettingStatusResult findCourseWeightSettingStatus(Long teachCourseId) {
        CourseWeightSettingStatusResult result = new CourseWeightSettingStatusResult();
        TeachCourse teachCourse = this.getById(teachCourseId);
        if (Objects.isNull(teachCourse)) {
            throw ExceptionResultEnum.ERROR.exception("教学课程不存在");
        }
        Boolean everSettingStatus = teachCourse.getWeightSetting();
        Boolean currenSettingStatus = courseWeightService.count(
                new QueryWrapper<CourseWeight>().lambda().eq(CourseWeight::getTeachCourseId, teachCourseId)) > 0;
        Long createId = teachCourse.getCreateId();
        result.setEverSettingStatus(everSettingStatus);
        result.setCurrentSettingStatus(currenSettingStatus);
        result.setCourseCreateId(createId);
        return result;
    }

    @Override
    public List<DictionaryResult> findTeacherCourseInfo(Long userId) {
        SysUser sysUser = sysUserService.getById(userId);
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }
        List<Long> courseCodeList = this.list(
                new QueryWrapper<TeachCourse>().lambda().eq(TeachCourse::getUserId, userId)).stream().map(TeachCourse::getCourseId).distinct().collect(Collectors.toList());
        return courseCodeList.stream().flatMap(e -> {
            BasicCourse basicCourse = basicCourseService.getById(e);
            DictionaryResult dictionaryResult = new DictionaryResult();
            dictionaryResult.setCode(basicCourse.getCode());
            dictionaryResult.setName(basicCourse.getName());
            return Stream.of(dictionaryResult);
        }).collect(Collectors.toList());
    }

    @Override
    public CourseResult listUserCourse(String param, Long teachingRoomId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        CourseResult courseResult = new CourseResult();
        // 教学课程
        List<BasicCourse> teachCourseList = this.listByUserId(sysUser.getId());
        courseResult.setTeachCourseList(teachCourseList);

        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(teachingRoomId);
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(BasicCourse::getTeachingRoomId, orgIds).eq(BasicCourse::getEnable, true);
        if (StringUtils.isNotBlank(param)) {
            queryWrapper.lambda().and(i -> i.like(BasicCourse::getCode, SystemConstant.translateSpecificSign(param))
                    .or().like(BasicCourse::getName, SystemConstant.translateSpecificSign(param)));
        }
        List<BasicCourse> basicCourseList = basicCourseService.list(queryWrapper);
        courseResult.setBasicCourseList(basicCourseList);

        return courseResult;
    }

    @Override
    public List<BasicCourse> listByUserId(Long userId) {
        QueryWrapper<TeachCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TeachCourse::getUserId, userId);
        List<TeachCourse> list = this.list(queryWrapper);
        List<BasicCourse> basicCourseList = list.stream().map(m -> basicCourseService.getById(m.getCourseId())).collect(Collectors.toList());
        return basicCourseList;
    }

    @Override
    public void batchCreateTeachCourse(Long schoolId, Long examId, List<BasicExamStudent> basicExamStudentList) {
        Map<Long, List<BasicExamStudent>> listMap = basicExamStudentList.stream().filter(e -> Objects.nonNull(e.getTeacherId())).collect(Collectors.groupingBy(BasicExamStudent::getTeacherId));
        for (Map.Entry<Long, List<BasicExamStudent>> entry : listMap.entrySet()) {
            TeachCourseSelectParam teachCourseSelectParam = new TeachCourseSelectParam();
            teachCourseSelectParam.setSchoolId(schoolId);
            teachCourseSelectParam.setExamId(examId);
            teachCourseSelectParam.setUserId(entry.getKey());
            teachCourseSelectParam.setCourseIds(entry.getValue().stream().map(BasicExamStudent::getCourseId).distinct().collect(Collectors.toList()));
            selectCreate(teachCourseSelectParam);
        }
    }
}
