package com.qmth.teachcloud.common.entity;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.enums.BasicExamStudentStatusEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.apache.batik.transcoder.TranscodingHints;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 考生字典表
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
@ApiModel(value = "BasicExamStudent对象", description = "考生字典表")
public class BasicExamStudent extends BaseEntity {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "学校id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long schoolId;

    @ApiModelProperty(value = "学期id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long semesterId;

    @ApiModelProperty(value = "考试id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long examId;

    @ApiModelProperty(value = "课程id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long courseId;

    @ApiModelProperty(value = "教学班")
    private String teachClassName;

    @ApiModelProperty(value = "行政班")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String className;

    @ApiModelProperty(value = "试卷编号")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String paperNumber;

    @ApiModelProperty(value = "姓名")
    private String studentName;

    @ApiModelProperty(value = "学号")
    private String studentCode;

    @ApiModelProperty(value = "座位号")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String siteNumber;

    @ApiModelProperty(value = "学院")
    private String collegeName;

    @ApiModelProperty(value = "专业")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String majorName;

    @ApiModelProperty(value = "任课教师id")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Long teacherId;

    @ApiModelProperty(value = "考试开始时间")
    private Long examStartTime;

    @ApiModelProperty(value = "考试结束时间")
    private Long examEndTime;

    @ApiModelProperty("考点(校区)")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String examPlace;

    @ApiModelProperty("考场(考试教室)")
    private String examRoom;

    @ApiModelProperty("基础字段")
    private String requiredFields;

    @ApiModelProperty("扩展字段")
    private String extendFields;

    @ApiModelProperty("状态")
    private BasicExamStudentStatusEnum status;

    @ApiModelProperty("基础字段集合")
    @TableField(exist = false)
    private List<CodeNameEnableValue> requiredFieldList;

    @ApiModelProperty("扩展字段集合")
    @TableField(exist = false)
    private List<CodeNameEnableValue> extendFieldList;

    @ApiModelProperty("状态")
    @TableField(exist = false)
    private String statusDisplay;

    public Long getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Long schoolId) {
        this.schoolId = schoolId;
    }

    public Long getSemesterId() {
        return semesterId;
    }

    public void setSemesterId(Long semesterId) {
        this.semesterId = semesterId;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public String getTeachClassName() {
        return teachClassName;
    }

    public void setTeachClassName(String teachClassName) {
        this.teachClassName = teachClassName;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getStudentName() {
        return studentName;
    }

    public void setStudentName(String studentName) {
        this.studentName = studentName;
    }

    public String getStudentCode() {
        return studentCode;
    }

    public void setStudentCode(String studentCode) {
        this.studentCode = studentCode;
    }

    public String getSiteNumber() {
        return siteNumber;
    }

    public void setSiteNumber(String siteNumber) {
        this.siteNumber = siteNumber;
    }

    public String getCollegeName() {
        return collegeName;
    }

    public void setCollegeName(String collegeName) {
        this.collegeName = collegeName;
    }

    public String getMajorName() {
        return majorName;
    }

    public void setMajorName(String majorName) {
        this.majorName = majorName;
    }

    public Long getTeacherId() {
        return teacherId;
    }

    public void setTeacherId(Long teacherId) {
        this.teacherId = teacherId;
    }

    public Long getExamStartTime() {
        return examStartTime;
    }

    public void setExamStartTime(Long examStartTime) {
        this.examStartTime = examStartTime;
    }

    public Long getExamEndTime() {
        return examEndTime;
    }

    public void setExamEndTime(Long examEndTime) {
        this.examEndTime = examEndTime;
    }

    public String getExamPlace() {
        return examPlace;
    }

    public void setExamPlace(String examPlace) {
        this.examPlace = examPlace;
    }

    public String getExamRoom() {
        return examRoom;
    }

    public void setExamRoom(String examRoom) {
        this.examRoom = examRoom;
    }

    public String getRequiredFields() {
        return requiredFields;
    }

    public void setRequiredFields(String requiredFields) {
        this.requiredFields = requiredFields;
    }

    public String getExtendFields() {
        return extendFields;
    }

    public void setExtendFields(String extendFields) {
        this.extendFields = extendFields;
    }

    public BasicExamStudentStatusEnum getStatus() {
        return status;
    }

    public void setStatus(BasicExamStudentStatusEnum status) {
        this.status = status;
    }

    public List<CodeNameEnableValue> getRequiredFieldList() {
        return StringUtils.isNotBlank(this.requiredFields) ? JSON.parseArray(this.requiredFields, CodeNameEnableValue.class) : Collections.emptyList();
    }

    public void setRequiredFieldList(List<CodeNameEnableValue> requiredFieldList) {
        this.requiredFieldList = requiredFieldList;
    }

    public List<CodeNameEnableValue> getExtendFieldList() {
        return StringUtils.isNotBlank(this.extendFields) ? JSON.parseArray(this.extendFields, CodeNameEnableValue.class) : Collections.emptyList();
    }

    public void setExtendFieldList(List<CodeNameEnableValue> extendFieldList) {
        this.extendFieldList = extendFieldList;
    }

    public String getStatusDisplay() {
        return this.status != null ? this.status.getName() : "";
    }

    public void setStatusDisplay(String statusDisplay) {
        this.statusDisplay = statusDisplay;
    }
}
